import subprocess
import os
import uuid

import questionary

from ..lib.io import save

def switch_user_prompt_temporary(config) -> dict:
    users = []

    for user in config["users"]:
        if 'username' in user and 'email' in user:
            users.append(f"{user['username']} ({user['email']}) | {user['url']} {"(current)" if user["current"] else ""}")
        elif 'linkTo' in user:
            linked_user = next((alias for alias in config["aliases"] if alias.get('id') == user['linkTo']), None)
            if linked_user:
                users.append(f"{linked_user['username']} ({linked_user['email']}) | {user['url']} ({"current, " if user["current"] else ""}alias)")
            else:
                users.append(f"Unknown User | {user['url']} ({"current, " if user["current"] else ""}alias)")
        else:
            users.append(f"Unknown User | {user['url']} {"(current)" if user["current"] else ""}")

    users.append("Exit")
    select_user = questionary.select("Select Account", choices=users).unsafe_ask()

    if select_user != "Exit":
        selected_index = users.index(select_user)
        selected_user = config["users"][selected_index]
        env = os.environ.copy()
        if selected_user.get("linkTo"):
            linked_user = next((alias for alias in config["aliases"] if alias.get('id') == selected_user['linkTo']), None)
            username = linked_user["username"]
            email = linked_user["email"]
        else:
            username = selected_user["username"]
            email = selected_user["email"]
        
        env["GIT_COMMITTER_NAME"] = username
        env["GIT_AUTHOR_NAME"] = username
        env["GIT_COMMITTER_EMAIL"] = email
        env["GIT_AUTHOR_EMAIL"] = email
        env["GCM_NAMESPACE"] = selected_user["id"]
        return env
    else:
        return None

def switch_user_prompt(config, local: bool):
    users = []

    for user in config["users"]:
        if 'username' in user and 'email' in user:
            users.append(f"{user['username']} ({user['email']}) | {user['url']} {"(current)" if user["current"] else ""}")
        elif 'linkTo' in user:
            linked_user = next((alias for alias in config["aliases"] if alias.get('id') == user['linkTo']), None)
            if linked_user:
                users.append(f"{linked_user['username']} ({linked_user['email']}) | {user['url']} ({"current, " if user["current"] else ""}alias)")
            else:
                users.append(f"Unknown User | {user['url']} ({"current, " if user["current"] else ""}alias)")
        else:
            users.append(f"Unknown User | {user['url']} {"(current)" if user["current"] else ""}")

    users.append("Exit")
    select_user = questionary.select("Select Account", choices=users).unsafe_ask()

    if select_user != "Exit":
        selected_index = users.index(select_user)
        selected_user = config["users"][selected_index]
        if selected_user.get("id") is None:
            selected_user["id"] = str(uuid.uuid4())
        subprocess.run(f"git config {"--global" if not local else "--local"} credential.namespace {selected_user["id"]}")
        if selected_user.get("linkTo"):
            linked_user = next((alias for alias in config["aliases"] if alias.get('id') == selected_user['linkTo']), None)
            subprocess.run(f'git config {"--global" if not local else "--local"} user.name "{linked_user["username"]}"', shell=True)
            subprocess.run(f'git config {"--global" if not local else "--local"} user.email "{linked_user["email"]}"', shell=True)
        else:
            subprocess.run(f'git config {"--global" if not local else "--local"} user.name "{selected_user["username"]}"', shell=True)
            subprocess.run(f'git config {"--global" if not local else "--local"} user.email "{selected_user["email"]}"', shell=True)
        for user in config["users"]:
            if selected_user["id"] == user["id"]:
                selected_user["current"] = True
            else:
                user["current"] = False
    else:
        pass
    return save(config)



import questionary

from ..lib.user import check_user_exists, add_user
from ..lib.io import save

def add_user_prompt(config):
    while True:
        remote = questionary.text("Enter Remote URL").unsafe_ask()
        if remote != "":
            break
        else:
            print("Enter a value.")

    link_alias = questionary.confirm(
        "Link to an existing alias?", default=False
    ).unsafe_ask()
    link_to = None
    if not link_alias:
        while True:
            username = questionary.text("Enter Git username").unsafe_ask()
            if username != "":
                break
            else:
                print("Enter a value.")
        while True:
            email = questionary.text("Enter Git email").unsafe_ask()
            if email != "":
                break
            else:
                print("Enter a value.")
    else:
        aliases = [f"{user['username']} ({user['email']})" for user in config["aliases"]]
        aliases.append("No link to aliases")
        select_alias = questionary.select("Select aliases to delete", choices=aliases).unsafe_ask()
        if select_alias != "No link to aliases":
            selected_index = aliases.index(select_alias)
            selected_alias = config["aliases"][selected_index]
            link_to = selected_alias["id"]
            username = None
            email = None
        else:
            while True:
                username = questionary.text("Enter Git username").unsafe_ask()
                if username != "":
                    break
                else:
                    print("Enter a value.")
            while True:
                email = questionary.text("Enter Git email").unsafe_ask()
                if email != "":
                    break
                else:
                    print("Enter a value.")

    confirm = questionary.confirm(
        "Would you like to add with this information?", default=False
    ).unsafe_ask()
    if not confirm:
        print("No users were added.")
    else:
        is_exists = check_user_exists(config, remote, username, email, link_to)
        if is_exists:
            print("This user already exists.")
        else:
            config = add_user(config, remote, username, email, link_to)
            config = save(config)
            print("User added!")
    return config


import questionary

from ..lib.io import save
from ..lib.alias import add_alias


def add_alias_prompt(config) -> dict:
    while True:
        username = questionary.text("Enter username").unsafe_ask()
        if username != "":
            break
        else:
            print("Enter a value.")
    while True:
        email = questionary.text("Enter email").unsafe_ask()
        if email != "":
            break
        else:
            print("Enter a value.")
    confirm = questionary.confirm(
        "Would you like to add with this information?", default=False
    ).unsafe_ask()
    if not confirm:
        print("No aliases were added.")
    else:
        config = add_alias(config, username, email)
        config = save(config)
        print("Alias added!")
    return config



import typer


from ..commands import add_user_prompt, delete_user_prompt, switch_user_prompt
from ..lib.io import init_config

app = typer.Typer(name="user")

@app.command("switch", short_help="Change current user. Use --local to apply this change only locally.")
def switch_user(local: bool = False):
    config = init_config()
    try:
        switch_user_prompt(config, local)
    except KeyboardInterrupt:
        pass


@app.command("add", short_help="Add a new user.")
def add_user():
    config = init_config()
    try:
        add_user_prompt(config)
    except KeyboardInterrupt:
        pass
    
@app.command("delete", short_help="Delete an existing user.")
def delete_user():
    config = init_config()
    try:
        delete_user_prompt(config)
    except KeyboardInterrupt:
        pass


import questionary
from ..lib.io import save

def delete_alias_prompt(config):
    aliases = [f"{user['username']} ({user['email']})" for user in config["aliases"]]
    aliases.append("Exit")
    select_alias = questionary.select("Select aliases to delete", choices=aliases).unsafe_ask()

    if select_alias != "Exit":
        selected_index = aliases.index(select_alias)
        selected_alias = config["aliases"][selected_index]

        linked_users = [user for user in config["users"] if user.get('linkTo') == selected_alias['id']]
        
        if linked_users:
            urls = [user['url'] for user in linked_users if 'url' in user]
            url_list = ', '.join(urls) if urls else "Unknown Host"
            warning_message = (
                f"Warning: The following urls are linked to this alias and will be affected:\n"
                f"{url_list}\n"
                "Do you still want to proceed with the deletion?"
            )
            confirm = questionary.confirm(warning_message, default=False).unsafe_ask()
        else:
            confirm = questionary.confirm(
                "Are you sure you want to delete it? This operation cannot be undone.",
                default=False,
            ).unsafe_ask()

        if confirm:
            config["users"] = [user for user in config["users"] if user.get("linkTo") != selected_alias["id"]]
            config["aliases"].remove(selected_alias)
            config = save(config)
            print(f"Alias {selected_alias['username']} has been removed.")
        else:
            print("No aliases were removed.")
    else:
        print("No aliases were removed.")
    
    return config



import argparse
import warnings

import questionary

from .commands import switch_user_prompt, add_user_prompt, add_alias_prompt, delete_user_prompt, delete_alias_prompt
from .lib.io import init_config


parser = argparse.ArgumentParser(description="Example script.")
parser.add_argument('--local', action='store_true')
args = parser.parse_args()

def run():
    warnings.warn("The prl command is deprecated; please use the paral command.", DeprecationWarning)
    config = init_config()
    try:
        while True:
            command = questionary.select(
                "What do you do?",
                choices=["Switch User", "Add User", "Add Aliases", "Delete User", "Delete Aliases", "Exit"],
            ).unsafe_ask()

            if command == "Switch User":
                config = switch_user_prompt(config, args.local)
            elif command == "Add User":
                config = add_user_prompt(config)
            elif command == "Delete User":
                config = delete_user_prompt(config)
            elif command == "Add Aliases":
                config = add_alias_prompt(config)
            elif command == "Delete Aliases":
                config = delete_alias_prompt(config)
            elif command == "Exit":
                break
            else:
                print("Unknown Command.")
    except KeyboardInterrupt:
        print("Bye!")


import typer


from ..commands import add_alias_prompt, delete_alias_prompt
from ..lib.io import init_config

app = typer.Typer(name="alias")

@app.command("add", short_help="Add a new alias.")
def add_alias():
    config = init_config()
    try:
        add_alias_prompt(config)
    except KeyboardInterrupt:
        pass

@app.command("delete", short_help="Delete an existing alias.")
def delete_alias():
    config = init_config()
    try:
        delete_alias_prompt(config)
    except KeyboardInterrupt:
        pass


from .add_alias import add_alias_prompt
from .add_users import add_user_prompt
from .delete_alias import delete_alias_prompt
from .delete_users import delete_user_prompt
from .switch_user import switch_user_prompt

__all__ = [
    "switch_user_prompt",
    "add_alias_prompt",
    "add_user_prompt",
    "delete_alias_prompt",
    "delete_user_prompt"
]


import questionary

from ..lib.io import save


def delete_user_prompt(config):
    users = []

    for user in config["users"]:
        if 'username' in user and 'email' in user:
            users.append(f"{user['username']} ({user['email']}) | {user['url']}")
        elif 'linkTo' in user:
            linked_user = next((alias for alias in config["aliases"] if alias.get('id') == user['linkTo']), None)
            if linked_user:
                users.append(f"{linked_user['username']} ({linked_user['email']}) | {user['url']} (alias)")
            else:
                users.append(f"Unknown User | {user['url']} (alias)")
        else:
            users.append(f"Unknown User | {user['url']}")

    users.append("Exit")
    select_user = questionary.select("Select users to delete", choices=users).unsafe_ask()

    if select_user != "Exit":
        selected_index = users.index(select_user)
        selected_user = config["users"][selected_index]

        confirm = questionary.confirm(
            "Are you sure you want to delete it? This operation cannot be undone.",
            default=False,
        ).unsafe_ask()
        if confirm:
            config["users"].remove(selected_user)
            config = save(config)
            print(f"User {selected_user['username']} has been removed.")
        else:
            print("No users were removed.")
    else:
        print("No users were removed.")
    return config



import typer

from ..commands.switch_user import switch_user_prompt, switch_user_prompt_temporary
from ..lib import shell
from ..lib.io import init_config
from . import alias, user

app = typer.Typer(no_args_is_help=True, short_help="An Git User/Credential Switcher")
app.add_typer(alias.app)
app.add_typer(user.app)

@app.command("env", short_help="Open a temporary session with the any Git informations.")
def environment():
    config = init_config()
    env = switch_user_prompt_temporary(config)
    if env:
        shell.launch(env)

@app.command("switch", short_help="Change current user. Use --local to apply this change only locally.")
def switch_user(local: bool = False):
    config = init_config()
    try:
        switch_user_prompt(config, local)
    except KeyboardInterrupt:
        pass